package ru.landgrafhomyak.db.skeleton1.api.module

import ru.landgrafhomyak.db.skeleton1.api.LowLevelApi
import ru.landgrafhomyak.db.skeleton1.api.statement.RawStatement
import ru.landgrafhomyak.db.skeleton1.api.statement._Statement
import ru.landgrafhomyak.db.skeleton1.api.runtime.InputRow
import ru.landgrafhomyak.db.skeleton1.api.table.Table


public interface ModuleUpgradeScope<nmUE : Any, omUE : Any, RK : Any> : ModuleConstructorScope<nmUE, RK> {
	// Table -------------------------------------------------------------------
	public fun <tUE : Any> keepTable(table: Table<tUE, omUE, RK>): Table<tUE, nmUE, RK>
	public fun <tUE : Any> keepAndRenameTable(
		table: Table<tUE, omUE, RK>,
		newNamespace: Namespace<nmUE, RK>? = null, newName: String
	): Table<tUE, nmUE, RK>

	public fun <ntUE : Any, otUE : Any> upgradeTable(
		oldTable: Table<otUE, omUE, RK>,
		upgrade: Table.Upgrade<ntUE, otUE, RK>
	): Table<ntUE, nmUE, RK>

	public fun <ntUE : Any, otUE : Any> upgradeAndRenameTable(
		table: Table<otUE, omUE, RK>,
		newNamespace: Namespace<nmUE, RK>? = null, newName: String,
		upgrade: Table.Upgrade<ntUE, otUE, RK>
	): Table<ntUE, nmUE, RK>

	public fun deleteTable(table: Table<*, omUE, RK>)
	public fun <tUE : Any> deleteTableAfterModuleUpgrade(table: Table<tUE, omUE, RK>): Table<tUE, nmUE, RK>

	public fun exportTable(table: Table<*, omUE, RK>)
	public fun <tUE : Any> importTable(
		table: Table<tUE, *, RK>,
		namespace: Namespace<nmUE, RK> = this.rootNs, name: String
	): Table<tUE, nmUE, RK>

	public fun <ntUE : Any, otUE : Any> importAndUpgradeTable(
		table: Table<otUE, *, RK>,
		namespace: Namespace<nmUE, RK> = this.rootNs, name: String,
		upgrade: Table.Upgrade<ntUE, otUE, RK>
	): Table<ntUE, nmUE, RK>

	// Object ------------------------------------------------------------------

	public fun <oUE : Any?> keepModuleScopedObject(obj: ModuleScopedObject<oUE, omUE, RK>): ModuleScopedObject<oUE, nmUE, RK>
	public fun <oUE : Any?> renameModuleScopedObject(
		obj: ModuleScopedObject<oUE, omUE, RK>,
		newNamespace: Namespace<nmUE, RK>? = null, newName: String
	): ModuleScopedObject<oUE, nmUE, RK>

	public fun <oUE : Any?> deleteModuleScopedObject(obj: ModuleScopedObject<oUE, omUE, RK>)
	public fun <oUE : Any?> deleteModuleScopedObjectAfterModuleUpgrade(obj: ModuleScopedObject<oUE, omUE, RK>): ModuleScopedObject<oUE, nmUE, RK>

	public fun exportModuleScopedObject(obj: ModuleScopedObject<*, omUE, RK>)
	public fun <oUE : Any> importModuleScopedObject(
		obj: ModuleScopedObject<oUE, *, RK>,
		namespace: Namespace<nmUE, RK> = this.rootNs, name: String
	): ModuleScopedObject<oUE, nmUE, RK>


	// Module ------------------------------------------------------------------

	public fun <smUE : Any> keepModule(module: Module<smUE, RK>): Module<smUE, RK>
	public fun <smUE : Any> keepAndMoveModule(oldModule: Module<smUE, RK>, newNs: Namespace<nmUE, RK>): Module<smUE, RK>

	public fun <smUE : Any> upgradeModule(oldModule: Module<*, RK>, template: Module.Template<smUE>): Module<smUE, RK>
	public fun <smUE : Any> upgradeAndMoveModule(
		oldModule: Module<*, RK>,
		newNamespace: Namespace<nmUE, RK>,
		template: Module.Template<smUE>
	): Module<smUE, RK>

	public fun deleteModule(oldModule: Module<*, RK>)
	public fun <smUE : Any> deleteModuleAfterUpgrade(oldModule: Module<smUE, RK>): Module<smUE, RK>

	public fun exportModule(table: Module<*, RK>)

	public fun <smUE : Any> importModule(
		module: Module<smUE, RK>,
		namespace: Namespace<nmUE, RK>,
	): Module<smUE, RK>

	public fun <nsmUE : Any, osmUE : Any> importAndUpgradeModule(
		module: Module<osmUE, RK>,
		namespace: Namespace<nmUE, RK>,
		upgrade: Module.Upgrade<osmUE, nsmUE, RK>
	): Module<nsmUE, RK>

	// Statements --------------------------------------------------------------

	public fun addStatementAfterUpgrade(stmt: _Statement.Void2Void<*, RK>)
	public fun addStatementAfterUpgrade(stmt: RawStatement.Void2Void.ImplementationsProvider<*>)

	@LowLevelApi
	public fun <sUE : Any> _addStatementAfterUpgrade(stmt: _Statement.Params2Void<sUE, RK>): InputRow._Scope<sUE, RK, Unit>

	@LowLevelApi
	public fun <sUE : Any> _addStatementAfterUpgrade(stmt: RawStatement.Params2Void.ImplementationsProvider<sUE>): InputRow._Scope<sUE, RK, Unit>
}